const std = @import("std");
const c = @import("ffi.zig").c;
const dbus = @import("dbus.zig");
const Image = @import("Image.zig");

const Entry = union(enum) {
    loading,
    cached: Image,
};

data: std.StringHashMap(Entry),

const Self = @This();

pub fn init(alloc: std.mem.Allocator) Self {
    return .{
        .data = std.StringHashMap(Entry).init(alloc),
    };
}

pub fn deinit(self: *Self) void {
    self.data.deinit();
    self.* = undefined;
}

/// Will grab the image if its already cached,
/// otherwise will start a DBus request to load it, reloading the view
/// once the image is received.
pub fn get(
    self: *Self,
    url: [:0]const u8,
    proxy: *c.nhdbus,
    icon_data_alloc: std.mem.Allocator,
) !?Image {
    const result = try self.data.getOrPut(url);
    if (result.found_existing) {
        switch (result.value_ptr.*) {
            .loading => return null,
            .cached => |img| return img,
        }
    }

    std.log.info("Requesting icon `{s}`", .{url});

    try dbus.requestIcon(proxy, self, url, icon_data_alloc);
    result.value_ptr.* = .loading;
    return null;
}
